gc()
library(methods)
library(optparse)
library(Seurat)
library(readr)
library(fBasics)
library(pastecs)
library(qusage)
library(RColorBrewer)
library(monocle)
library(dplyr)
library(viridis)
library(readxl)

source("../r.scripts/sc-TissueMapper.R")

#Create folder structure
setwd("../")
if (!dir.exists("./analysis")){
  dir.create("./analysis")
}
if (!dir.exists("./analysis/qc")){
  dir.create("./analysis/qc")
}
if (!dir.exists("./analysis/qc/cc")){
  dir.create("./analysis/qc/cc")
}
if (!dir.exists("./analysis/tSNE")){
  dir.create("./analysis/tSNE")
}
if (!dir.exists("./analysis/tSNE/pre.stress")){
  dir.create("./analysis/tSNE/pre.stress")
}
if (!dir.exists("./analysis/pca")){
  dir.create("./analysis/pca")
}
if (!dir.exists("./analysis/pca/stress")){
  dir.create("./analysis/pca/stress")
}
if (!dir.exists("./analysis/violin")){
  dir.create("./analysis/violin")
}
if (!dir.exists("./analysis/violin/stress")){
  dir.create("./analysis/violin/stress")
}
if (!dir.exists("./analysis/table")){
  dir.create("./analysis/table")
}
if (!dir.exists("./analysis/tSNE/post.stress")){
  dir.create("./analysis/tSNE/post.stress")
}
if (!dir.exists("./analysis/cor")){
  dir.create("./analysis/cor")
}
if (!dir.exists("./analysis/tSNE/lin")){
  dir.create("./analysis/tSNE/lin")
}
if (!dir.exists("./analysis/tSNE/epi")){
  dir.create("./analysis/tSNE/epi")
}
if (!dir.exists("./analysis/tSNE/n-epi")){
  dir.create("./analysis/tSNE/n-epi")
}
if (!dir.exists("./analysis/tSNE/st")){
  dir.create("./analysis/tSNE/st")
}
if (!dir.exists("./analysis/tSNE/leu")){
  dir.create("./analysis/tSNE/leu")
}
if (!dir.exists("./analysis/tSNE/lymphoid")){
  dir.create("./analysis/tSNE/lymphoid")
}
if (!dir.exists("./analysis/tSNE/myeloid")){
  dir.create("./analysis/tSNE/myeloid")
}
if (!dir.exists("./analysis/tSNE/merge")){
  dir.create("./analysis/tSNE/merge")
}
if (!dir.exists("./analysis/pca/ne")){
  dir.create("./analysis/pca/ne")
}
if (!dir.exists("./analysis/tSNE/ne")){
  dir.create("./analysis/tSNE/ne")
}
if (!dir.exists("./analysis/violin/ne")){
  dir.create("./analysis/violin/ne")
}
if (!dir.exists("./analysis/tSNE/FINAL")){
  dir.create("./analysis/tSNE/FINAL")
}
if (!dir.exists("./analysis/deg")){
  dir.create("./analysis/deg")
}
if (!dir.exists("./analysis/cca")){
  dir.create("./analysis/cca")
}
if (!dir.exists("./analysis/diy")){
  dir.create("./analysis/diy")
}
if (!dir.exists("./analysis/pseudotime")){
  dir.create("./analysis/pseudotime")
}

#Retrieve command-line options
option_list=list(
  make_option("--p",action="store",default="DPrF",type='character',help="Project Name"),
  make_option("--g",action="store",default="ALL",type='character',help="Group To analyze"),
  make_option("--lg",action="store",default=500,type='integer',help="Threshold for cells with minimum genes"),
  make_option("--hg",action="store",default=3000,type='integer',help="Threshold for cells with maximum genes"),
  make_option("--lm",action="store",default=0,type='numeric',help="Threshold for cells with minimum %mito genes"),
  make_option("--hm",action="store",default=0.1,type='numeric',help="Threshold for cells with maximum %mito genes"),
  make_option("--lx",action="store",default=0.2,type='numeric',help="x low threshold for hvg selection"),
  make_option("--hx",action="store",default=5,type='numeric',help="x high threshold for hvg selection"),
  make_option("--ly",action="store",default=1,type='numeric',help="y low threshold for hvg selection"),
  make_option("--cc",action="store",default=TRUE,type='logical',help="Scale cell cycle?"),
  make_option("--cca",action="store",default=50,type='integer',help="Number of CCAs to cacluate"),
  make_option("--acca",action="store",default=30,type='integer',help="Number of CCAs to align"),
  make_option("--pc",action="store",default=50,type='integer',help="Number of PCs to cacluate"),
  make_option("--res.prestress",action="store",default=1,type='numeric',help="Resolution to cluster, pre-stress"),
  make_option("--st",action="store",default=TRUE,type='logical',help="Remove stressed cells?"),
  make_option("--stg",action="store",default="dws",type='character',help="Geneset to use for stress ID"),
  make_option("--cut.stress",action="store",default=0.9,type='numeric',help="Cutoff for stress score"),
  make_option("--res.poststress",action="store",default=0.25,type='numeric',help="Resolution to cluster, post-stress"),
  make_option("--cut.ne",action="store",default=0.999,type='numeric',help="Cutoff for NE score")
)
opt=parse_args(OptionParser(option_list=option_list))
rm(option_list)
if (opt$lm==0){opt$lm=-Inf}

load("./analysis/sc10x.Pd.Rda")
load("./analysis/sc10x.Pb.Rda")
load("./analysis/sc10x.Pc.Rda")

for (i in c("Pb","Pc","Pd")){
  if (!dir.exists(paste0("./analysis/tSNE/post.stress/",i))){
    dir.create(paste0("./analysis/tSNE/post.stress/",i))
  }
  if (!dir.exists(paste0("./analysis/tSNE/lin/",i))){
    dir.create(paste0("./analysis/tSNE/lin/",i))
  }
  if (!dir.exists(paste0("./analysis/tSNE/epi/",i))){
    dir.create(paste0("./analysis/tSNE/epi/",i))
  }
  if (!dir.exists(paste0("./analysis/tSNE/n-epi/",i))){
    dir.create(paste0("./analysis/tSNE/n-epi/",i))
  }
  if (!dir.exists(paste0("./analysis/tSNE/st/",i))){
    dir.create(paste0("./analysis/tSNE/st/",i))
  }
  if (!dir.exists(paste0("./analysis/tSNE/leu/",i))){
    dir.create(paste0("./analysis/tSNE/leu/",i))
  }
  if (!dir.exists(paste0("./analysis/tSNE/lymphoid/",i))){
    dir.create(paste0("./analysis/tSNE/lymphoid/",i))
  }
  if (!dir.exists(paste0("./analysis/tSNE/myeloid/",i))){
    dir.create(paste0("./analysis/tSNE/myeloid/",i))
  }
  
  sc10x <- get(paste0("sc10x.",i))
  
  sc10x <- scCluster(sc10x,pc.use=opt$acca,res.use=opt$res.poststress,folder=paste0("post.stress/",i),red="cca.aligned")
  
#   gene.set1 <- read_delim("./genesets/genes.deg.Epi.csv",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
#   gene.set1 <- gene.set1[1]
#   gene.set1 <- as.list(gene.set1)
#   names(gene.set1) <- "Epi"
#   gene.set <- c(gene.set1)
#   gene.set1 <- read_delim("./genesets/genes.deg.St.csv",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
#   gene.set1 <- gene.set1[1]
#   gene.set1 <- as.list(gene.set1)
#   names(gene.set1) <- "St"
#   gene.set <- c(gene.set,gene.set1)
  gene.set1 <- read_delim("./genesets/genes.deg.BE.csv",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
  gene.set1 <- gene.set1[1]
  gene.set1 <- as.list(gene.set1)
  names(gene.set1) <- "Epi"
  gene.set <- c(gene.set1) #replaced gene.set <- c(gene.set,gene.set1) so pan-epi and pan-st not correlated
  gene.set1 <- read_delim("./genesets/genes.deg.LE.csv",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
  gene.set1 <- gene.set1[1]
  gene.set1 <- as.list(gene.set1)
  names(gene.set1) <- "Epi"
  gene.set <- c(gene.set,gene.set1)
  gene.set1 <- read_delim("./genesets/genes.deg.OE1.csv",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
  gene.set1 <- gene.set1[1]
  gene.set1 <- as.list(gene.set1)
  names(gene.set1) <- "Epi"
  gene.set <- c(gene.set,gene.set1)
  gene.set1 <- read_delim("./genesets/genes.deg.OE2.csv",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
  gene.set1 <- gene.set1[1]
  gene.set1 <- as.list(gene.set1)
  names(gene.set1) <- "Epi"
  gene.set <- c(gene.set,gene.set1)
  gene.set1 <- read_delim("./genesets/genes.deg.Endo.csv",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
  gene.set1 <- gene.set1[1]
  gene.set1 <- as.list(gene.set1)
  names(gene.set1) <- "Endo"
  gene.set <- c(gene.set,gene.set1)
  gene.set1 <- read_delim("./genesets/genes.deg.SM.csv",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
  gene.set1 <- gene.set1[1]
  gene.set1 <- as.list(gene.set1)
  names(gene.set1) <- "FMSt"
  gene.set <- c(gene.set,gene.set1)
  gene.set1 <- read_delim("./genesets/genes.deg.Fib.csv",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
  gene.set1 <- gene.set1[1]
  gene.set1 <- as.list(gene.set1)
  names(gene.set1) <- "FMSt"
  gene.set <- c(gene.set,gene.set1)
  genes.leu <- read_excel("genesets/40425_2017_215_MOESM1_ESM.xlsx",sheet="S3. Candidate markers")
  leu <- as.list(unique(genes.leu[,2]))$Cell
  leu <- leu[-c(9,17,20)]
  leu.l <- leu[-c(1,3:4,7:8,13:18,21,20:30)]
  #leu.lin <- c("Myeloid","Lymphoid","Lymphoid","Lymphoid","Myeloid","Myeloid","Lymphoid","Myeloid","Myeloid","Myeloid","Myeloid","Myeloid","Lymphoid","Lymphoid","Lymphoid","Myeloid","Lymphoid","Lymphoid","Lymphoid","Lymphoid","Lymphoid","Lymphoid","Lymphoid","Lymphoid","Lymphoid","Lymphoid","Lymphoid")
  leu.lin <- c("Lymphoid","Myeloid","Myeloid","Myeloid","Myeloid","Myeloid","Myeloid","Lymphoid")  
  #genes.leu <- genes.leu[unlist(genes.leu[,2]) %in% unlist(leu),]
  genes.leu.l <- genes.leu[unlist(genes.leu[,2]) %in% unlist(leu.l),]
  genes.leu.l[nrow(genes.leu.l)+1,] <- list("MS4A7","Macrophages",0)
genes.leu.l[nrow(genes.leu.l)+1,] <- list("CD14","Macrophages",0)
genes.leu.l[nrow(genes.leu.l)+1,] <- list("CD86","Macrophages",0)
genes.leu.l[nrow(genes.leu.l)+1,] <- list("S100A9","Neutrophils",0)
genes.leu.l[nrow(genes.leu.l)+1,] <- list("EREG","Neutrophils",0)
genes.leu.l[nrow(genes.leu.l)+1,] <- list("S100A8","Neutrophils",0)
genes.leu.l[nrow(genes.leu.l)+1,] <- list("FCN1","Neutrophils",0)
  #gene.set1 <- split(genes.leu[,1], genes.leu[,2])
  gene.set1 <- split(genes.leu.l[,1], genes.leu.l[,2])
  gene.set1 <- lapply(gene.set1,unname)
  gene.set1 <- lapply(gene.set1,unlist)
  names(gene.set1) <- leu.lin
  gene.set <- c(gene.set,gene.set1)
  rm(gene.set1)
  gc()
  min.all <- min(table(sc10x@meta.data[,paste0("res",opt$res.poststress)]))
  results <- scQuSAGE.nocorplot(sc10x,gs=gene.set,res.use=opt$res.poststress,ds=min.all,nm="Pop",folder=paste0("lin/",i))
  sc10x <- results[[1]]
  results.cor.Lin <- results[[2]]
  results.clust.Lin.id <- results[[3]]
  rm(results)
  rm(gene.set)
  
  res.pop <- 1

  sc10x <- SetAllIdent(object=sc10x,id="Pop")

  try({
    sc10x.Epi <- scSubset(sc10x,i="Pop",g="Epi")
    sc10x.Epi <- SetAllIdent(object=sc10x.Epi,id=paste0("res",opt$res.poststress))
    sc10x.Epi <- StashIdent(object=sc10x.Epi,save.name=paste0("old.res",opt$res.poststress))
    sc10x.Epi <- scCluster(sc10x.Epi,pc.use=opt$acca,res.use=res.pop,folder=paste0("epi/",i),red="cca.aligned")
    assign(paste0("sc10x.Epi.",i,".analized"),sc10x.Epi)
  })

  try({
    sc10x.FMSt <- scSubset(sc10x,i="Pop",g="FMSt")
    sc10x.FMSt <- SetAllIdent(object=sc10x.FMSt,id=paste0("res",opt$res.poststress))
    sc10x.FMSt <- StashIdent(object=sc10x.FMSt,save.name=paste0("old.res",opt$res.poststress))
    sc10x.FMSt <- scCluster(sc10x.FMSt,pc.use=opt$acca,res.use=res.pop,folder=paste0("st/",i),red="cca.aligned")
    assign(paste0("sc10x.FMSt.",i,".analized"),sc10x.FMSt)
  })

  try({
    sc10x.Lymphoid <- scSubset(sc10x,i="Pop",g="Lymphoid")
    sc10x.Lymphoid <- SetAllIdent(object=sc10x.Lymphoid,id=paste0("res",opt$res.poststress))
    sc10x.Lymphoid <- StashIdent(object=sc10x.Lymphoid,save.name=paste0("old.res",opt$res.poststress))
    sc10x.Lymphoid <- scCluster(sc10x.Lymphoid,pc.use=opt$acca,res.use=res.pop,folder=paste0("lymphoid/",i),red="cca.aligned")
    assign(paste0("sc10x.Lymphoid.",i,".analized"),sc10x.Lymphoid)
  })

  try({
    sc10x.Myeloid <- scSubset(sc10x,i="Pop",g="Myeloid")
    sc10x.Myeloid <- SetAllIdent(object=sc10x.Myeloid,id=paste0("res",opt$res.poststress))
    sc10x.Myeloid <- StashIdent(object=sc10x.Myeloid,save.name=paste0("old.res",opt$res.poststress))
    sc10x.Myeloid <- scCluster(sc10x.Myeloid,pc.use=opt$acca,res.use=res.pop,folder=paste0("myeloid/",i),red="cca.aligned")
    assign(paste0("sc10x.Myeloid.",i,".analized"),sc10x.Myeloid)
  })
  
  assign(paste0("sc10x.",i,".analized"),sc10x)
  assign(paste0("results.cor.Lin.",i),results.cor.Lin)
  assign(paste0("results.clust.Lin.id.",i),results.clust.Lin.id)

  rm(sc10x)
  rm(sc10x.Epi)
  rm(sc10x.FMSt)
  rm(sc10x.Lymphoid)
  rm(sc10x.Myeloid)
  rm(sc10x.Leu)
  rm(results.cor.Lin)
  rm(results.clust.Lin.id)
  rm(min.all)
}
rm(i)

save(list=ls(pattern="sc10x.*.Pd.analized"),file="./analysis/sc10x.Pd.analized.Rda")
save(list=ls(pattern="sc10x.*.Pb.analized"),file="./analysis/sc10x.Pb.analized.Rda")
save(list=ls(pattern="sc10x.*.Pc.analized"),file="./analysis/sc10x.Pc.analized.Rda")
save.image(file="./analysis/sc10x.analized.RData")

